from os import getenv
from typing import Dict
import logging

import boto3
import boto3.session

sqspy_logger = logging.getLogger("sqs_py")


class Base:
    QUEUE_VISIBILITY_TIMEOUT = "600"

    def __init__(self, **kwargs):
        aws_access_key_id = kwargs.get("aws_access_key_id")
        aws_secret_access_key = kwargs.get("aws_secret_access_key")
        profile_name = kwargs.get("profile_name")
        endpoint_url = kwargs.get("endpoint_url")
        region_name = kwargs.get("region_name")
        self._session = boto3.session.Session(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            profile_name=profile_name,
            region_name=region_name,
        )
        self._sqs = self._session.resource(
            "sqs",
            region_name=region_name,
            endpoint_url=endpoint_url,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
        )
        sqspy_logger.debug("Initialised SQS resource")

    def get_queue(self, queue_data: Dict):
        queue_name = queue_data.get("name")
        queue_visibility: str = queue_data.get("visibility_timeout") or self.QUEUE_VISIBILITY_TIMEOUT

        queue_url = queue_data.get("url")
        queue_name = queue_data.get("name")
        if queue_url:
            return self._sqs.Queue(queue_url)

        return self._sqs.get_queue_by_name(QueueName=queue_name)




import json
import logging
import os

from ._base import Base

sqspy_logger = logging.getLogger("sqs_py")


class Producer(Base):
    def __init__(self, queue_name: str = None, queue_url: str = None, **kwargs):
        queue = kwargs.get("queue")
        if not any([queue, queue_name, queue_url]):
            raise ValueError(
                "One of `queue`, `queue_name` or `queue_url` should be provided"
            )
        super().__init__(**kwargs)
        queue_data: Dict = {
            "name": queue_name,
            "url": queue_url,
            "visibility_timeout": kwargs.get("visibility_timeout"),
        }
        self._queue = queue or self.get_or_create_queue(
            queue_data, create_queue=kwargs.get("create_queue"),
        )
        if self.queue is None:
            raise ValueError(
                "No queue found with name or URL provided, or "
                "application did not have permission to create one."
            )
        self._queue_name = self._queue.url.split("/")[-1]

    @property
    def queue(self):
        return self._queue

    @property
    def queue_name(self):
        return self._queue_name

    def publish(self, message, **kwargs):
        sqspy_logger.info(f"Sending message to queue {self.queue_name}.")
        return self._queue.send_message(MessageBody=json.dumps(message), **kwargs)



MAJOR = 0
MINOR = 0
PATCH = 1
SUFFIX = ""

NAME = "sqs_py"

AUTHOR = MAINTAINER = {
    "name": "wengjiyao",
    "email": "w.j@live.ca",
}

AUTHORS = MAINTAINERS = (
    f"{AUTHOR.get('name')} <{AUTHOR.get('email')}>",
)

__version__ = VERSION = f"{MAJOR}.{MINOR}.{PATCH}{SUFFIX}"



# To use a consistent encoding
from codecs import open
from pathlib import Path
from setuptools import setup, find_packages

import sqs_py.about as about


here = Path.cwd()
readme = here / "README.md"

# Get the long description from the README file
with readme.open() as f:
    long_description = f.read()


setup(
    name=about.NAME,
    version=about.VERSION,
    description="AWS SQS utility package for producing and consuming messages",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/wengjiyao/sqs_py",
    author=about.AUTHOR.get("name"),
    author_email=about.AUTHOR.get("email"),
    license="MIT",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3.7",
    ],
    # What does your project relate to?
    keywords="aws sqs messages producer/consumer",
    packages=find_packages(),
    install_requires=["boto3"],
)



from abc import ABCMeta, abstractmethod
from time import sleep
from typing import Dict, List
import json
import logging
import os
import sys

from ._base import Base
from .producer import Producer

sqspy_logger = logging.getLogger("sqs_py")


class Consumer(Base):
    __metaclass__ = ABCMeta
    WAIT_TIME = 0
    POLL_INTERVAL = 60
    MAX_MESSAGES_COUNT = 1

    def __init__(self, queue_name: str = None, queue_url: str = None, **kwargs):
        queue = kwargs.get("queue")
        if not any([queue, queue_name, queue_url]):
            raise ValueError(
                "One of `queue`, `queue_name` or `queue_url` should be provided"
            )
        super().__init__(**kwargs)
        queue_data: Dict = {
            "name": queue_name,
            "url": queue_url,
            "visibility_timeout": kwargs.get("visibility_timeout"),
        }
        error_queue_data: Dict = {
            "name": kwargs.get("error_queue"),
            "url": kwargs.get("error_queue_url"),
            "visibility_timeout": kwargs.get("error_visibility_timeout"),
        }
        self._poll_interval: int = int(kwargs.get("interval", self.POLL_INTERVAL))
        self._message_attribute_names: List = kwargs.get("message_attribute_names", [])
        self._attribute_names: List = kwargs.get("attribute_names", [])
        self._wait_time: int = int(kwargs.get("wait_time", self.WAIT_TIME))
        self._max_number_of_messages: int = int(
            kwargs.get("max_number_of_messages", self.MAX_MESSAGES_COUNT)
        )
        self._force_delete: bool = kwargs.get("force_delete", False)
        self._queue = queue or self.get_queue(queue_data)
        if self.queue is None:
            raise ValueError(
                "No queue found with name or URL provided, or "
                "application did not have permission to create one."
            )
        self._queue_name = self._queue.url.split("/")[-1]
        self._error_queue = None
        if error_queue_data.get("name") or error_queue_data.get("url"):
            self._error_queue = Producer(
                queue_name=error_queue_data.get("name"),
                queue_url=error_queue_data.get("url"),
                queue=self.get_queue(error_queue_data),
            )
            #self._error_queue_name = self._error_queue.url.split("/")[-1]
            self._error_queue_name = self._error_queue.queue_name

    @property
    def queue(self):
        return self._queue

    @property
    def queue_name(self):
        return self._queue_name

    def poll_messages(self):
        while True:
            messages = self._queue.receive_messages(
                AttributeNames=self._attribute_names,
                MessageAttributeNames=self._message_attribute_names,
                WaitTimeSeconds=self._wait_time,
                MaxNumberOfMessages=self._max_number_of_messages,
            )
            if not messages:
                sqspy_logger.debug(
                    f"No messages were fetched for {self.queue_name}. "
                    f"Sleeping for {self._poll_interval} seconds."
                )
                sleep(self._poll_interval)
                continue
            sqspy_logger.info(
                f"{len(messages)} messages received for {self.queue_name}"
            )
            break
        return messages

    def _start_listening(self):
        while True:
            messages = self.poll_messages()
            for message in messages:
                m_body = message.body
                message_attribs = message.message_attributes
                attribs: Dict = message.attributes
                # catch problems with malformed JSON, usually a result
                # of someone writing poor JSON directly in the AWS
                # console
                try:
                    body = json.loads(m_body)
                except:
                    sqspy_logger.warning(
                        f"Unable to parse message - JSON is not formatted properly. "
                        f"Received message: {m_body}"
                    )
                    continue
                try:
                    if self._force_delete:
                        message.delete()
                        self.handle_message(body, message_attribs, attribs)
                    else:
                        self.handle_message(body, message_attribs, attribs)
                        message.delete()
                except Exception as ex:
                    # need exception logtype to log stack trace
                    sqspy_logger.exception(ex)
                    if self._error_queue:
                        exc_type, exc_obj, exc_tb = sys.exc_info()
                        sqspy_logger.info("Pushing exception to error queue")
                        self._error_queue.publish(
                            {
                                "exception_type": str(exc_type),
                                "error_message": str(ex.args),
                            }
                        )

    def listen(self):
        sqspy_logger.info(f"Listening to queue {self.queue_name}")
        if self._error_queue:
            sqspy_logger.info(f"Using error queue {self._error_queue_name}")
        self._start_listening()

    @abstractmethod
    def handle_message(self, body, attributes, messages_attributes):
        raise NotImplementedError("Implement this function in subclass.")
